@@ -1,6 +1,6 @@
 /*
 
- Copyright (C) 2011 Alexis Bienvenue <paamc@passoire.fr>
+ Copyright (C) 2011-2012 Alexis Bienvenue <paamc@passoire.fr>
 
  This file is part of Auto-Multiple-Choice
 
@@ -45,6 +45,9 @@
     #if CV_MINOR_VERSION >= 1
        #define OPENCV_21 1
     #endif
+    #if CV_MINOR_VERSION >= 3
+       #define OPENCV_23 1
+    #endif
   #endif
 #endif
 
@@ -73,32 +76,65 @@ void agrege(double x,double y,double* coins_x,double* coins_y) {
   AGREGE_POINT(-,<,3)
 }
 
-void load_image(IplImage** src,char *filename,double threshold=0.6,int view=0) {
-  if((*src=cvLoadImage(filename, CV_LOAD_IMAGE_GRAYSCALE))!= 0) {
-    double max;
-    cvMinMaxLoc(*src,NULL,&max);
-    cvSmooth(*src,*src,CV_GAUSSIAN,3,3,1);
-    cvThreshold(*src,*src, max*threshold, 255, CV_THRESH_BINARY_INV );
+void load_image(IplImage** src,char *filename,
+		int ignore_red,double threshold=0.6,int view=0) {
+  IplImage* color;
+  double max;
 
-    if((*src)->origin==1) {
-      printf(": Image flip\n");
-      cvFlip(*src,NULL,0);
+  if(ignore_red) {
+    if((color=cvLoadImage(filename,
+#ifdef OPENCV_23
+			  CV_LOAD_IMAGE_ANYCOLOR
+#else
+			  CV_LOAD_IMAGE_UNCHANGED
+#endif
+			  ))!= NULL) {
+      printf("loaded\n");
+      if(color->nChannels>=3) {
+	/* keeps only red channel */
+	cvSetImageCOI(color,3);
+	*src=cvCreateImage( cvGetSize(color), color->depth, 1 );
+	cvCopy(color,*src);
+	cvReleaseImage(&color);
+      } else if(color->nChannels!=1) {
+	printf("! LOAD : Scan file with 2 channels [%s]\n",filename);
+	return;
+      } else {
+	*src=color;
+      }
+    } else {
+      printf("! LOAD : Error loading scan file in ANYCOLOR [%s]\n",filename);
+      return;
     }
-
   } else {
-    printf("! LOAD : Error loading scan file [%s]\n",filename);
+    if((*src=cvLoadImage(filename, CV_LOAD_IMAGE_GRAYSCALE))==NULL) {
+      printf("! LOAD : Error loading scan file in GRAYSCALE [%s]\n",filename);
+      return;
+    }
+  }
+
+  cvMinMaxLoc(*src,NULL,&max);
+  cvSmooth(*src,*src,CV_GAUSSIAN,3,3,1);
+  cvThreshold(*src,*src, max*threshold, 255, CV_THRESH_BINARY_INV );
+  
+  if((*src)->origin==1) {
+    printf(": Image flip\n");
+    cvFlip(*src,NULL,0);
   }
 }
 
 void pre_traitement(IplImage* src,int lissage_trous,int lissage_poussieres) {
-  IplConvKernel* kernel=cvCreateStructuringElementEx(3,3,1,1,CV_SHAPE_RECT);
+  IplConvKernel* trous=cvCreateStructuringElementEx(1+2*lissage_trous,1+2*lissage_trous,
+						    lissage_trous,lissage_trous,CV_SHAPE_ELLIPSE);
+  IplConvKernel* poussieres=cvCreateStructuringElementEx(1+2*lissage_poussieres,1+2*lissage_poussieres,
+							 lissage_poussieres,lissage_poussieres,CV_SHAPE_ELLIPSE);
 
   printf("Morph: +%d -%d\n",lissage_trous,lissage_poussieres);
-  cvDilate(src,src,kernel,lissage_trous);
-  cvErode(src,src,kernel,lissage_trous+lissage_poussieres);
-  cvDilate(src,src,kernel,lissage_poussieres);
+  cvMorphologyEx(src,src,NULL,trous,CV_MOP_CLOSE);
+  cvMorphologyEx(src,src,NULL,poussieres,CV_MOP_OPEN);
 
-  cvReleaseStructuringElement(&kernel);
+  cvReleaseStructuringElement(&trous);
+  cvReleaseStructuringElement(&poussieres);
 }
 
 typedef struct {
@@ -208,7 +244,7 @@ void calage(IplImage* src,IplImage* illustr,
 		 1+(int)((target_min+target_max)/2 /8));
 
   if(view==2) {
-    *dst=cvCreateImage( cvGetSize(src), 8, 3 );
+    *dst=cvCreateImage( cvGetSize(src), IPL_DEPTH_8U, 3 );
     cvConvertImage(src,*dst);
     cvNot(*dst,*dst);
   }
@@ -292,9 +328,12 @@ void calage(IplImage* src,IplImage* illustr,
   cvClearMemStorage(storage);
 }
 
+#define CLOSER(pointa,pointb,cn,dist,delta) dist=delta*(pointb.cn-pointa.cn);pointa.cn+=dist;pointb.cn-=dist;
+
 void deplace(int i,int j,double delta,point *coins) {
-  coins[i].x+=delta*(coins[j].x-coins[i].x);
-  coins[i].y+=delta*(coins[j].y-coins[i].y);
+  double d;
+  CLOSER(coins[i],coins[j],x,d,delta);
+  CLOSER(coins[i],coins[j],y,d,delta);
 }
 
 void restreint(int *x,int *y,int tx,int ty) {
@@ -304,35 +343,29 @@ void restreint(int *x,int *y,int tx,int ty) {
   if(*y>=ty) *y=ty-1;
 }
 
-int check_zooms_dir(int student, int page, char *zooms_dir=NULL,int log=0) {
+int check_zooms_dir(int student, char *zooms_dir=NULL,int log=0) {
   int ok=1;
-  static char* zoom_page_dir=NULL;
   struct stat zd;
   
   if(student>=0) {
-    if(asprintf(&zoom_page_dir,"%s/%d-%d",zooms_dir,student,page)>0) {
-      if(stat(zoom_page_dir,&zd)!=0) {
-	if(errno==ENOENT) {
-	  if(mkdir(zoom_page_dir,0755)!=0) {
-	    ok=0;
-	    printf("! ZOOMDC : Zoom dir creation error %d : %s\n",errno,zoom_page_dir);
-	  } else {
-	    printf(": Zoom dir created %s\n",zoom_page_dir);
-	  }
-	} else {
+    if(stat(zooms_dir,&zd)!=0) {
+      if(errno==ENOENT) {
+	if(mkdir(zooms_dir,0755)!=0) {
 	  ok=0;
-	  printf("! ZOOMDS : Zoom dir stat error %d : %s\n",errno,zoom_page_dir);
+	  printf("! ZOOMDC : Zoom dir creation error %d : %s\n",errno,zooms_dir);
+	} else {
+	  printf(": Zoom dir created %s\n",zooms_dir);
 	}
       } else {
-	if(!S_ISDIR(zd.st_mode)) {
-	  ok=0;
-	  printf("! ZOOMDP : Zoom dir is not a directory : %s\n",zoom_page_dir);
-	}
-      }	  
+	ok=0;
+	printf("! ZOOMDS : Zoom dir stat error %d : %s\n",errno,zooms_dir);
+      }
     } else {
-      printf("! ZOOMDN : Zoom dir name error\n");
-      ok=0;
-    }
+      if(!S_ISDIR(zd.st_mode)) {
+	ok=0;
+	printf("! ZOOMDP : Zoom dir is not a directory : %s\n",zooms_dir);
+      }
+    }	  
   } else {
     ok=0;
     if(log) {
@@ -397,8 +430,8 @@ void mesure_case(IplImage *src,IplImage *illustr,
 
   /* box reduction */
   delta=(1-prop)/2;
-  deplace(0,2,delta,coins);deplace(2,0,delta,coins);
-  deplace(1,3,delta,coins);deplace(3,1,delta,coins);
+  deplace(0,2,delta,coins);
+  deplace(1,3,delta,coins);
 
   /* output points used for mesuring */
   for(i=0;i<4;i++) {
@@ -428,11 +461,11 @@ void mesure_case(IplImage *src,IplImage *illustr,
     if(zooms_dir!=NULL && student>=0) {
 
       /* check if directory is present, or ceate it */
-      ok=check_zooms_dir(student,page,zooms_dir,0);
+      ok=check_zooms_dir(student,zooms_dir,0);
 
       /* save zoom file */
       if(ok) {
-	if(asprintf(&zoom_file,"%s/%d-%d/%d-%d.png",zooms_dir,student,page,question,answer)>0) {
+	if(asprintf(&zoom_file,"%s/%d-%d.png",zooms_dir,question,answer)>0) {
 	  printf(": Saving zoom to %s\n",zoom_file);
 	  printf(": Z=(%d,%d)+(%d,%d)\n",
 		 z_xmin,z_ymin,z_xmax-z_xmin,z_ymax-z_ymin);
@@ -444,6 +477,8 @@ void mesure_case(IplImage *src,IplImage *illustr,
 #endif
 			 )!=1) {
 	    printf("! ZOOMS : Zoom save error\n");
+	  } else {
+	    printf("ZOOM %d-%d.png\n",question,answer);
 	  }
 	  
 	  cvResetImageROI(illustr);
@@ -525,23 +560,34 @@ int main( int argc, char** argv )
   char *zooms_dir=NULL;
   int view=0;
   int post_process_image=0;
+  int ignore_red=0;
 
 #if OPENCV_20
   int save_options[3]={CV_IMWRITE_JPEG_QUALITY,75,0};
 #endif
 
   // Options
+  // -x tx : gives the width of the original subject
+  // -y ty : gives the height of the opriginal subject
+  // -d d  : gives the diameter of the corner marks on the original subject
+  // -p dp : gives the tolerance above mark diameter (fraction of the diameter)
+  // -m dm : gives the tolerance below mark diameter
+  // -t th : gives the threshold to convert to black&white
+  // -o file : gives output file name for detected layout report image
+  // -v / -P : asks for marks detection debugging image report
 
   char c;
-  while ((c = getopt (argc, argv, "x:y:d:i:p:m:o:vP")) != -1) {
+  while ((c = getopt (argc, argv, "x:y:d:i:p:m:t:o:vPr")) != -1) {
     switch (c) {
     case 'x': taille_orig_x=atof(optarg);break; 
     case 'y': taille_orig_y=atof(optarg);break; 
     case 'd': dia_orig=atof(optarg);break; 
     case 'p': tol_plus=atof(optarg);break;
     case 'm': tol_moins=atof(optarg);break;
+    case 't': threshold=atof(optarg);break;
     case 'o': out_image_file=strdup(optarg);break;
     case 'v': view=1;break;
+    case 'r': ignore_red=1;break;
     case 'P': post_process_image=1;view=2;break;
     }
   }
@@ -568,9 +614,6 @@ int main( int argc, char** argv )
     } else if(strncmp(commande,"zooms ",6)==0) {
       free(zooms_dir);
       zooms_dir=strdup(commande+6);
-    } else if(sscanf(commande,"createzd %d %d",&student,&page)==2) {
-      check_zooms_dir(student,page,zooms_dir,1);
-      student=-1;
     } else if(strncmp(commande,"load ",5)==0) {
       free(scan_file);
       scan_file=strdup(commande+5);
@@ -588,7 +631,7 @@ int main( int argc, char** argv )
 	}
       }
 
-      load_image(&src,scan_file,threshold,view);
+      load_image(&src,scan_file,ignore_red,threshold,view);
 
       src_calage=cvCloneImage(src);
       calage(src_calage,illustr,
@@ -630,13 +673,18 @@ int main( int argc, char** argv )
 	transfo.b=-transfo.b;
 	transfo.c=-transfo.c;
 	transfo.d=-transfo.d;
-	transfo.e=src->width-transfo.e;
-	transfo.f=src->height-transfo.f;
+	transfo.e=(src->width-1)-transfo.e;
+	transfo.f=(src->height-1)-transfo.f;
 	
 	if(src!=NULL) cvFlip(src,NULL,-1);
 	if(illustr!=NULL) cvFlip(illustr,NULL,-1);
 	if(dst!=NULL) cvFlip(dst,NULL,-1);
 
+	for(i=0;i<4;i++) {
+	  coins_x[i]=(src->width-1)-coins_x[i];
+	  coins_y[i]=(src->height-1)-coins_y[i];
+	}
+
 	upside_down=0;
 
 	printf("Transfo:\na=%f\nb=%f\nc=%f\nd=%f\ne=%f\nf=%f\n",
