@@ -1,6 +1,6 @@
 #! /usr/bin/perl
 #
-# Copyright (C) 2009,2011 Alexis Bienvenue <paamc@passoire.fr>
+# Copyright (C) 2009,2011-2012 Alexis Bienvenue <paamc@passoire.fr>
 #
 # This file is part of Auto-Multiple-Choice
 #
@@ -25,16 +25,17 @@ use AMC::Gui::Avancement;
 
 use Module::Load;
 
-#use encoding 'utf8';
+use_amc_plugins();
 
 my $module='CSV';
 my $output='';
 
+my $data_dir='';
 my $fich_notes='';
 my $fich_assoc='';
 my $fich_noms='';
 my $noms_encodage='utf-8';
-my $noms_identifiant='';
+my $csv_build_name='';
 my @o_out=();
 my $debug='';
 my $sort='n';
@@ -47,17 +48,16 @@ my $rtl='';
 GetOptions("module=s"=>\$module,
 	   "sort=s"=>\$sort,
 	   "useall=s"=>\$useall,
-	   "fich-notes=s"=>\$fich_notes,
-	   "fich-assoc=s"=>\$fich_assoc,
+	   "data=s"=>\$data_dir,
 	   "fich-noms=s"=>\$fich_noms,
+	   "csv-build-name=s"=>\$csv_build_name,
 	   "noms-encodage=s"=>\$noms_encodage,
-	   "noms-identifiant=s"=>\$noms_identifiant,
 	   "rtl!"=>\$rtl,
 	   "option-out=s"=>\@o_out,
 	   "output|o=s"=>\$output,
 	   "debug=s"=>\$debug,
 	   );
-	   
+
 set_debug($debug);
 
 debug "Parameters: ".join(" ",map { "<$_>" } @ARGV_ORIG);
@@ -65,25 +65,18 @@ debug "Parameters: ".join(" ",map { "<$_>" } @ARGV_ORIG);
 load("AMC::Export::$module");
 $ex = "AMC::Export::$module"->new();
 
-my %sorting=('l'=>['n:_LINE_'],
-	     'm'=>['n:_NOTE_','s:_NOM_'],
-	     'i'=>['n:_ID_'],
-	     'n'=>['s:_NOM_','n:_LINE_','n:_ID_'],
-    );
-
 $ex->set_options("sort",
-		 "keys"=>$sorting{lc($1)}) if($sort =~ /^\s*([lmin])/i);
+		 "keys"=>$sort);
 
 $ex->set_options("fich",
-		 "notes"=>$fich_notes,
-		 "association"=>$fich_assoc,
+		 "datadir"=>$data_dir,
 		 "noms"=>$fich_noms,
 		 );
 
 $ex->set_options("noms",
 		 "encodage"=>$noms_encodage,
-		 "identifiant"=>$noms_identifiant,
 		 "useall"=>$useall,
+		 "identifiant"=>$csv_build_name,
 		 );
 
 $ex->set_options("out","rtl"=>$rtl);
@@ -94,4 +87,6 @@ for my $oo (@o_out) {
     }
 }
 
+debug "Exporting...";
+
 $ex->export($output);
