@@ -1,5 +1,5 @@
 #
-# Copyright (C) 2008-2010 Alexis Bienvenue <paamc@passoire.fr>
+# Copyright (C) 2008-2012 Alexis Bienvenue <paamc@passoire.fr>
 #
 # This file is part of Auto-Multiple-Choice
 #
@@ -51,10 +51,18 @@ sub new {
     $self->{'point.actuel'}=0;
 
     bless $self;
-    
+
     return($self);
 }
 
+sub clone {
+  my ($self)=@_;
+  my $s={'coins'=>[map { [@$_] ; } (@{$self->{'coins'}})],
+	 'droite'=>$self->{'droite'}};
+  bless $s;
+  return($s);
+}
+
 sub def_point_suivant {
     my ($self,$x,$y)=(@_);
     $self->{'coins'}->[$self->{'point.actuel'}++]=[$x,$y];
@@ -115,7 +123,7 @@ sub un_seul {
     if($t eq '') {
 	return($x);
     } elsif($t eq 'SCALAR') {
-	return($$x); 
+	return($$x);
     } elsif($t eq 'ARRAY') {
 	return($x->[0]);
     } elsif($t eq 'HASH') {
@@ -131,7 +139,7 @@ sub def_complete_xml {
     my ($self,$x)=(@_);
     $x=$x->{'coin'} if($x->{'coin'});
     $self->def_complete(map { (un_seul($x->{$_}->{'x'}),
-			       un_seul($x->{$_}->{'y'})) } 
+			       un_seul($x->{$_}->{'y'})) }
 			(1..4));
 
     return($self);
@@ -230,6 +238,13 @@ sub xml {
     return($x);
 }
 
+sub to_data {
+  my ($self,$capture,$zoneid,$type)=@_;
+  for my $i (0..3) {
+    $capture->set_corner($zoneid,$i+1,$type,@{$self->{'coins'}->[$i]});
+  }
+}
+
 # renvoie la commande a passer a AMC::Image pour mesurer le contenu de
 # la boite dans une image.
 
