@@ -1,6 +1,6 @@
 #! /usr/bin/perl -w
 #
-# Copyright (C) 2008-2011 Alexis Bienvenue <paamc@passoire.fr>
+# Copyright (C) 2008-2012 Alexis Bienvenue <paamc@passoire.fr>
 #
 # This file is part of Auto-Multiple-Choice
 #
@@ -21,6 +21,7 @@
 package AMC::Gui::PageArea;
 
 use Gtk2;
+use AMC::Basic;
 
 @ISA=("Gtk2::DrawingArea");
 
@@ -37,8 +38,13 @@ sub add_feuille {
     $self->{'tx'}=1;
     $self->{'ty'}=1;
 
+    $self->{'min_render_size'}=10;
+
     $self->{'case'}='';
     $self->{'coches'}='';
+    $self->{'editable'}=1;
+
+    $self->{'font'}=Pango::FontDescription->from_string("128");
 
     for (keys %oo) {
 	$self->{$_}=$oo{$_} if(defined($self->{$_}));
@@ -58,20 +64,43 @@ sub add_feuille {
 }
 
 sub set_image {
-    my ($self,$image,$lay,$coches)=@_;
+    my ($self,$image,$layinfo)=@_;
     $self->{'i-file'}=$image;
-    if($image && -f $image) {
-	$self->{'i-src'}=Gtk2::Gdk::Pixbuf->new_from_file($image);
+    if($image =~ /text:(.*)/) {
+      my $text=$1;
+
+      my $layout=$self->create_pango_layout($text);
+      my $colormap =$self->get_colormap;
+      $layout->set_font_description($self->{'font'});
+      my ($text_x,$text_y)=$layout->get_pixel_size();
+      my $pixmap=Gtk2::Gdk::Pixmap->new(undef,$text_x,$text_y,$colormap->get_visual->depth);
+      $pixmap->set_colormap($colormap);
+      $pixmap->draw_rectangle($self->style->bg_gc(GTK_STATE_NORMAL),TRUE,0,0,$text_x,$text_y);
+      $pixmap->draw_layout($self->style->fg_gc(GTK_STATE_NORMAL),0,0,$layout);
+      my $pixbuf=Gtk2::Gdk::Pixbuf->get_from_drawable($pixmap, $colormap,0,0,0,0, $text_x, $text_y);
+      $self->{'i-src'}=$pixbuf;
+    } elsif($image && -f $image) {
+      eval { $self->{'i-src'}=Gtk2::Gdk::Pixbuf->new_from_file($image); };
+      if($@) {
+	# Error loading scan...
+	$self->{'i-src'}='';
+      }
+    } elsif($image eq 'NONE') {
+      $self->{'i-src'}='';
     } else {
 	$self->{'i-src'}=Gtk2::Gdk::Pixbuf->new(GDK_COLORSPACE_RGB,0,8,40,10);
 	$self->{'i-src'}->fill(0x48B6FF);
     }
-    $self->{'lay'}=$lay;
-    $self->{'coches'}=$coches;
+    $self->{'layinfo'}=$layinfo;
     $self->{'modifs'}=0;
     $self->window->show;
 }
 
+sub get_image {
+   my ($self)=@_;
+   return($self->{'i-src'});
+ }
+
 sub modifs {
     my $self=shift;
     return($self->{'modifs'});
@@ -90,20 +119,35 @@ sub modif {
 sub choix {
   my ($self,$event)=(@_);
 
-  if($self->{'lay'}->{'case'}) {
-      
+  if(!$self->{'editable'}) {
+    return TRUE;
+  }
+
+  if($self->{'layinfo'}->{'block_message'}) {
+    my $dialog = Gtk2::MessageDialog
+      ->new_with_markup(undef,
+			'destroy-with-parent',
+			'error','ok',
+			$self->{'layinfo'}->{'block_message'});
+    $dialog->run;
+    $dialog->destroy;
+
+    return TRUE;
+  }
+
+  if($self->{'layinfo'}->{'box'}) {
+
       if ($event->button == 1) {
 	  my ($x,$y)=$event->coords;
-	  print "Click $x $y\n" if($self->{'debug'});
-	  for my $i (0..$#{$self->{'lay'}->{'case'}}) {
-	      
-	      my $case=$self->{'lay'}->{'case'}->[$i];
-	      if($x<=$case->{'xmax'}*$self->{'rx'} && $x>=$case->{'xmin'}*$self->{'rx'}
-		 && $y<=$case->{'ymax'}*$self->{'ry'} && $y>=$case->{'ymin'}*$self->{'ry'}) {
+	  debug "Click $x $y\n";
+	  for my $i (@{$self->{'layinfo'}->{'box'}}) {
+
+	      if($x<=$i->{'xmax'}*$self->{'rx'} && $x>=$i->{'xmin'}*$self->{'rx'}
+		 && $y<=$i->{'ymax'}*$self->{'ry'} && $y>=$i->{'ymin'}*$self->{'ry'}) {
 		  $self->{'modifs'}=1;
 
-		  print " -> box $i\n" if($self->{'debug'});
-		  $self->{'coches'}->[$i]=!$self->{'coches'}->[$i];
+		  debug " -> box $i\n";
+		  $i->{'ticked'}=!$i->{'ticked'};
 
 		  $self->window->show;
 	      }
@@ -115,7 +159,7 @@ sub choix {
 }
 
 sub expose_drawing {
-    my ($self,$evenement,@donnees)=@_;  
+    my ($self,$evenement,@donnees)=@_;
     my $r=$self->allocation();
 
     return() if(!$self->{'i-src'});
@@ -123,6 +167,8 @@ sub expose_drawing {
     $self->{'tx'}=$r->width;
     $self->{'ty'}=$r->height;
 
+    debug("Rendering target size: ".$self->{'tx'}."x".$self->{'ty'});
+
     my $sx=$self->{'tx'}/$self->{'i-src'}->get_width;
     my $sy=$self->{'ty'}/$self->{'i-src'}->get_height;
 
@@ -135,7 +181,10 @@ sub expose_drawing {
 	$sx=$self->{'tx'}/$self->{'i-src'}->get_width;
     }
 
-    return() if($self->{'tx'}<=0 || $self->{'ty'}<=0);
+    return() if($self->{'tx'}<$self->{'min_render_size'}
+		|| $self->{'ty'}<$self->{'min_render_size'});
+
+    debug("Rendering with SX=$sx SY=$sy");
 
     my $i=Gtk2::Gdk::Pixbuf->new(GDK_COLORSPACE_RGB,1,8,$self->{'tx'},$self->{'ty'});
 
@@ -150,74 +199,75 @@ sub expose_drawing {
 			   $self->{'ty'},
 			   'none',0,0);
 
-    if($self->{'lay'}->{'case'} || $self->{'lay'}->{'nom'}) {
-	my $case;
-	my $coche;
- 
-	$self->{'rx'}=$self->{'tx'}/$self->{'lay'}->{'tx'};
-	$self->{'ry'}=$self->{'ty'}/$self->{'lay'}->{'ty'};
+    debug "Done with rendering";
+
+    if($self->{'layinfo'}->{'box'} || $self->{'layinfo'}->{'namefield'}) {
+	my $box;
+
+	debug "Layout drawings...";
+
+	$self->{'rx'}=$self->{'tx'}/$self->{'layinfo'}->{'page'}->{'width'};
+	$self->{'ry'}=$self->{'ty'}/$self->{'layinfo'}->{'page'}->{'height'};
 
 	# layout drawings
 
 	if($self->{'marks'}) {
 	    $self->{'gc'}->set_foreground($self->{'colormark'});
-	    
-	    for $case (@{$self->{'lay'}->{'nom'}}) {
+
+	    for $box (@{$self->{'layinfo'}->{'namefield'}}) {
 		$self->window->draw_rectangle(
 		    $self->{'gc'},
 		    '',
-		    $case->{'xmin'}*$self->{'rx'},
-		    $case->{'ymin'}*$self->{'ry'},
-		    ($case->{'xmax'}-$case->{'xmin'})*$self->{'rx'},
-		    ($case->{'ymax'}-$case->{'ymin'})*$self->{'ry'}
+		    $box->{'xmin'}*$self->{'rx'},
+		    $box->{'ymin'}*$self->{'ry'},
+		    ($box->{'xmax'}-$box->{'xmin'})*$self->{'rx'},
+		    ($box->{'ymax'}-$box->{'ymin'})*$self->{'ry'}
 		    );
 	    }
 
-	    $case=$self->{'lay'}->{'coin'};
+	    $box=$self->{'layinfo'}->{'mark'};
 
-	    if($case) {
-		for my $i (1..4) {
-		    my $j=($i % 4)+1;
+	    if($box) {
+		for my $i (0..3) {
+		    my $j=(($i+1) % 4);
 		    $self->window->draw_line($self->{'gc'},
-					     $case->{$i}->{'x'}->[0]*$self->{'rx'},
-					     $case->{$i}->{'y'}->[0]*$self->{'ry'},
-					     $case->{$j}->{'x'}->[0]*$self->{'rx'},
-					     $case->{$j}->{'y'}->[0]*$self->{'ry'},
+					     $box->[$i]->{'x'}*$self->{'rx'},
+					     $box->[$i]->{'y'}*$self->{'ry'},
+					     $box->[$j]->{'x'}*$self->{'rx'},
+					     $box->[$j]->{'y'}*$self->{'ry'},
 			);
 		}
 	    }
 
-	    for my $case (@{$self->{'lay'}->{'chiffre'}}) {
+	    for my $box (@{$self->{'layinfo'}->{'digit'}}) {
 		$self->window->draw_rectangle(
 		    $self->{'gc'},
 		    '',
-		    $case->{'xmin'}*$self->{'rx'},
-		    $case->{'ymin'}*$self->{'ry'},
-		    ($case->{'xmax'}-$case->{'xmin'})*$self->{'rx'},
-		    ($case->{'ymax'}-$case->{'ymin'})*$self->{'ry'}
+		    $box->{'xmin'}*$self->{'rx'},
+		    $box->{'ymin'}*$self->{'ry'},
+		    ($box->{'xmax'}-$box->{'xmin'})*$self->{'rx'},
+		    ($box->{'ymax'}-$box->{'ymin'})*$self->{'ry'}
 		    );
-		
+
 	    }
-	    
+
 	}
 
 	## boxes drawings
 
 	$self->{'gc'}->set_foreground($self->{'color'});
 
-	for my $i (0..$#{$self->{'lay'}->{'case'}}) {
-	    $case=$self->{'lay'}->{'case'}->[$i];
-	    $coche=$self->{'coches'}->[$i];
-
+	for $box (@{$self->{'layinfo'}->{'box'}}) {
 	    $self->window->draw_rectangle(
-					  $self->{'gc'},
-					  $coche,
-					  $case->{'xmin'}*$self->{'rx'},
-					  $case->{'ymin'}*$self->{'ry'},
-					  ($case->{'xmax'}-$case->{'xmin'})*$self->{'rx'},
-					  ($case->{'ymax'}-$case->{'ymin'})*$self->{'ry'}
-					  );
-	    
+		$self->{'gc'},
+		$box->{'ticked'},
+		$box->{'xmin'}*$self->{'rx'},
+		$box->{'ymin'}*$self->{'ry'},
+		($box->{'xmax'}-$box->{'xmin'})*$self->{'rx'},
+		($box->{'ymax'}-$box->{'ymin'})*$self->{'ry'}
+		);
+
+	    debug "Done.";
 	}
     }
 }
